<?php

namespace Autos\Favourite\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Autos\Favourite\Http\Requests\FavouriteRequest;
use Autos\Favourite\Interfaces\FavouriteRepositoryInterface;
use Autos\Favourite\Models\Favourite;
use Autos\Favourite\Forms\Favourite as Form;

/**
 * APIController  class for favourite.
 */
class FavouriteAPIController extends BaseController
{

    /**
     * Initialize favourite resource controller.
     *
     * @param type FavouriteRepositoryInterface $favourite
     *
     * @return null
     */
    public function __construct(FavouriteRepositoryInterface $favourite)
    {
        parent::__construct();
        $this->repository = $favourite;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Autos\Favourite\Repositories\Criteria\FavouriteResourceCriteria::class);
    }

    /**
     * Display a list of favourite.
     *
     * @return Response
     */
    public function index(FavouriteRequest $request)
    {
        return $this->repository
            ->setPresenter(\Autos\Favourite\Repositories\Presenter\FavouritePresenter::class)
            ->paginate();
    }

    /**
     * Display favourite.
     *
     * @param Request $request
     * @param Model   $favourite
     *
     * @return Response
     */
    public function show(FavouriteRequest $request, Favourite $favourite)
    {
        return $favourite->setPresenter(\Autos\Favourite\Repositories\Presenter\FavouriteListPresenter::class);
        ;
    }

    /**
     * Create new favourite.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FavouriteRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('favourite::favourite.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('favourite/favourite/' . $favourite->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('favourite/favourite');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the favourite.
     *
     * @param Request $request
     * @param Model   $favourite
     *
     * @return Response
     */
    public function update(FavouriteRequest $request, Favourite $favourite)
    {
        try {
            $data = $request->all();

            $favourite->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('favourite::favourite.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('favourite/favourite/' . $favourite->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('favourite/favourite/' . $favourite->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the favourite.
     *
     * @param Model   $favourite
     *
     * @return Response
     */
    public function destroy(FavouriteRequest $request, Favourite $favourite)
    {
        try {
            $favourite->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('favourite::favourite.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('favourite/favourite/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('favourite/favourite/' . $favourite->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
