<?php

namespace Autos\Favourite\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Autos\Favourite\Http\Requests\FavouriteRequest;
use Autos\Favourite\Interfaces\FavouriteRepositoryInterface;
use Autos\Favourite\Models\Favourite;

/**
 * Resource controller class for favourite.
 */
class FavouriteResourceController extends BaseController
{

    /**
     * Initialize favourite resource controller.
     *
     * @param type FavouriteRepositoryInterface $favourite
     *
     * @return null
     */
    public function __construct(FavouriteRepositoryInterface $favourite)
    {
        parent::__construct();
        $this->repository = $favourite;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Autos\Favourite\Repositories\Criteria\FavouriteResourceCriteria::class);
    }

    /**
     * Display a list of favourite.
     *
     * @return Response
     */
    public function index(FavouriteRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Autos\Favourite\Repositories\Presenter\FavouritePresenter::class)
                ->$function();
        }

        $favourites = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('favourite::favourite.names'))
            ->view('favourite::favourite.index', true)
            ->data(compact('favourites', 'view'))
            ->output();
    }

    /**
     * Display favourite.
     *
     * @param Request $request
     * @param Model   $favourite
     *
     * @return Response
     */
    public function show(FavouriteRequest $request, Favourite $favourite)
    {

        if ($favourite->exists) {
            $view = 'favourite::favourite.show';
        } else {
            $view = 'favourite::favourite.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('favourite::favourite.name'))
            ->data(compact('favourite'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new favourite.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FavouriteRequest $request)
    {

        $favourite = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('favourite::favourite.name')) 
            ->view('favourite::favourite.create', true) 
            ->data(compact('favourite'))
            ->output();
    }

    /**
     * Create new favourite.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FavouriteRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $favourite                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('favourite::favourite.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('favourite/favourite/' . $favourite->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/favourite/favourite'))
                ->redirect();
        }

    }

    /**
     * Show favourite for editing.
     *
     * @param Request $request
     * @param Model   $favourite
     *
     * @return Response
     */
    public function edit(FavouriteRequest $request, Favourite $favourite)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('favourite::favourite.name'))
            ->view('favourite::favourite.edit', true)
            ->data(compact('favourite'))
            ->output();
    }

    /**
     * Update the favourite.
     *
     * @param Request $request
     * @param Model   $favourite
     *
     * @return Response
     */
    public function update(FavouriteRequest $request, Favourite $favourite)
    {
        try {
            $attributes = $request->all();

            $favourite->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('favourite::favourite.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('favourite/favourite/' . $favourite->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('favourite/favourite/' . $favourite->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the favourite.
     *
     * @param Model   $favourite
     *
     * @return Response
     */
    public function destroy(FavouriteRequest $request, Favourite $favourite)
    {
        try {

            $favourite->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('favourite::favourite.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('favourite/favourite/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('favourite/favourite/' . $favourite->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple favourite.
     *
     * @param Model   $favourite
     *
     * @return Response
     */
    public function delete(FavouriteRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('favourite::favourite.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('favourite/favourite'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/favourite/favourite'))
                ->redirect();
        }

    }

    /**
     * Restore deleted favourites.
     *
     * @param Model   $favourite
     *
     * @return Response
     */
    public function restore(FavouriteRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('favourite::favourite.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/favourite/favourite'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/favourite/favourite/'))
                ->redirect();
        }

    }

}
